Package: MultiStepSSAD
Type: Package
Title: Multiple-Steps Step-Stress Accelerated Degradation Modeling
Version: 0.1.0
Authors@R: c(
  person("Shikhar", "Tyagi", email = "shikhar1093tyagi@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-1606-0844")),
  person("Arvind", "Pandey", email = "arvindmzu@gmail.com", role = "aut"),
  person("Bhupendra", "Singh", email = "bhupendra.rana@gmail.com", role = "aut"),
  person("Vrijesh", "Tripathi", email = "vrijesh.tripathi@uwi.edu", role = "aut")
  )
Description: Implements statistical methods for multiple-steps step-stress accelerated degradation testing (SSADT) based on Wiener processes and Gamma processes as described by Pan and Balakrishnan (2010) <doi:10.1080/03610918.2010.496060>. Features maximum likelihood estimation (MLE), Bayesian Markov chain Monte Carlo (MCMC) sampling, Geweke convergence diagnostics, and reliability predictions under normal operating stress conditions.
License: GPL (>= 3)
Depends: R (>= 4.0.0)
Imports: stats, graphics, grDevices, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-31 01:10:31 UTC; shikhar tyagi
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Arvind Pandey [aut],
  Bhupendra Singh [aut],
  Vrijesh Tripathi [aut]
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-07 21:00:07 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,ssadfit)
S3method(predict,ssadfit)
S3method(print,ssadfit)
S3method(summary,ssadfit)
export(geweke_diag)
export(ssad_fit)
export(ssad_gamma)
export(ssad_reliability)
export(ssad_simulate)
export(ssad_wiener)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(utils,head)

#' Wiener Process Step-Stress Accelerated Degradation Model Estimation
#'
#' Fits a Wiener process step-stress accelerated degradation model using Maximum Likelihood Estimation (MLE)
#' or Bayesian Markov Chain Monte Carlo (MCMC) as described in Pan and Balakrishnan (2010).
#'
#' @param data A data frame containing inspection times in the first column and unit degradation measurements in subsequent columns,
#'   or a long-format data frame with columns \code{unit}, \code{time}, \code{degradation}.
#' @param stress_levels Vector of stress levels S = (S1, S2, ..., SK).
#' @param thresholds Vector of degradation thresholds (omega_1, ..., omega_K-1) at which stress levels are elevated.
#' @param model Acceleration model: \code{"arrhenius"} or \code{"power"}.
#' @param method Estimation method: \code{"mle"} or \code{"mcmc"}.
#' @param init Initial parameter values \code{c(a, b, sigma)}.
#' @param n_iter Number of MCMC iterations (default: 10000).
#' @param burnin Number of burn-in iterations (default: 2000).
#' @param thin Thinning interval for MCMC (default: 1).
#'
#' @return An object of class \code{"ssadfit"} containing fit results, parameter estimates, log-likelihood, MCMC samples (if applicable), and metadata.
#'
#' @references
#' Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
#' \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#'
#' @export
#' @examples
#' data(wiener_arrhenius)
#' fit_w <- ssad_wiener(
#'   data = wiener_arrhenius,
#'   stress_levels = c(45, 65, 85),
#'   thresholds = c(90, 160),
#'   model = "arrhenius",
#'   method = "mle"
#' )
#' print(fit_w)
ssad_wiener <- function(data, stress_levels = c(45, 65, 85), thresholds = c(90, 160),
                        model = c("arrhenius", "power"), method = c("mle", "mcmc"),
                        init = NULL, n_iter = 5000, burnin = 1000, thin = 1) {
  model <- match.arg(model)
  method <- match.arg(method)
  
  # Format data into unit list with times and degradation measurements
  parsed_data <- parse_ssad_data(data)
  units_data <- parsed_data$units
  
  K <- length(stress_levels)
  if (length(thresholds) != K - 1) {
    stop("Length of thresholds must equal length(stress_levels) - 1.")
  }
  
  # Estimate transition times tau_{i, k} for each unit i
  tau_mat <- estimate_ssad_taus(units_data, thresholds)
  
  eval_acc <- function(S, m_type, par) {
    if (is.function(m_type)) return(m_type(S, par))
    a <- par[1]; b <- par[2]
    if (m_type == "arrhenius") return(exp(a + b / (273 + S)))
    if (m_type == "power") return(a * (S^b))
    stop("Unsupported acceleration model.")
  }
  
  # Complete log-likelihood function
  loglik_fn <- function(par) {
    a <- par[1]; b <- par[2]; sigma <- par[3]
    if (is.na(sigma) || sigma <= 1e-6) return(-1e12)
    
    beta_k <- sapply(stress_levels, function(s) eval_acc(s, model, c(a, b)))
    if (any(is.na(beta_k)) || any(beta_k <= 1e-8)) return(-1e12)
    
    logL <- 0
    N <- length(units_data)
    
    # 1. Stress elevation time likelihood g(tau_i)
    d_omega <- c(thresholds[1], diff(thresholds))
    for (i in seq_len(N)) {
      tau_i <- tau_mat[i, ]
      d_tau <- c(tau_i[1], diff(tau_i))
      for (k in seq_len(K - 1)) {
        dtk <- max(d_tau[k], 1e-6)
        dok <- d_omega[k]
        bk <- beta_k[k]
        
        term1 <- 0.5 * log((dok^2) / (2 * pi * (dtk^3) * (sigma^2)))
        term2 <- -((bk * dtk - dok)^2) / (2 * (sigma^2) * dtk)
        logL <- logL + term1 + term2
      }
    }
    
    # 2. Degradation increment likelihood f(dx_{i,j})
    for (i in seq_len(N)) {
      u_df <- units_data[[i]]
      t_vec <- u_df$time
      x_vec <- u_df$degradation
      tau_i <- c(0, tau_mat[i, ], max(t_vec) * 10)
      
      M <- length(t_vec)
      for (j in seq_len(M - 1)) {
        t1 <- t_vec[j]; t2 <- t_vec[j + 1]
        dx <- x_vec[j + 1] - x_vec[j]
        dt <- t2 - t1
        
        # Calculate expected increment dm based on stress intervals
        dm <- 0
        for (k in seq_len(K)) {
          t_start_k <- tau_i[k]
          t_end_k <- tau_i[k + 1]
          overlap <- max(0, min(t2, t_end_k) - max(t1, t_start_k))
          if (overlap > 0) {
            dm <- dm + beta_k[k] * overlap
          }
        }
        
        term1 <- -0.5 * log(2 * pi * dt * (sigma^2))
        term2 <- -((dx - dm)^2) / (2 * (sigma^2) * dt)
        logL <- logL + term1 + term2
      }
    }
    
    if (is.na(logL) || is.nan(logL)) return(-1e12)
    return(logL)
  }
  
  if (is.null(init)) {
    if (model == "arrhenius") {
      init <- c(a = 5.0, b = -2500, sigma = 0.1)
    } else {
      init <- c(a = 1e-3, b = 1.2, sigma = 0.1)
    }
  }
  
  if (method == "mle") {
    neg_loglik <- function(p) -loglik_fn(p)
    opt <- stats::optim(init, neg_loglik, method = "Nelder-Mead", control = list(maxit = 2000))
    opt <- stats::optim(opt$par, neg_loglik, method = "BFGS", hessian = TRUE, control = list(maxit = 2000))
    
    par_mle <- opt$par
    names(par_mle) <- c("a", "b", "sigma")
    
    se_mle <- rep(NA, 3)
    if (!is.null(opt$hessian) && !any(is.na(opt$hessian))) {
      cov_mat <- tryCatch(solve(opt$hessian), error = function(e) NULL)
      if (!is.null(cov_mat)) {
        se_mle <- sqrt(pmax(diag(cov_mat), 0))
      }
    }
    names(se_mle) <- c("a", "b", "sigma")
    
    res <- list(
      par = par_mle,
      se = se_mle,
      loglik = -opt$value,
      process = "wiener",
      model = model,
      method = "mle",
      convergence = opt$convergence,
      stress_levels = stress_levels,
      thresholds = thresholds,
      tau_mat = tau_mat,
      units_data = units_data
    )
  } else { # MCMC
    # Metropolis-Hastings MCMC Sampler
    chain <- matrix(0, nrow = n_iter, ncol = 3)
    colnames(chain) <- c("a", "b", "sigma")
    curr_par <- init
    curr_loglik <- loglik_fn(curr_par)
    
    scale_vec <- abs(init) * 0.05
    scale_vec[scale_vec == 0] <- 0.01
    
    acc_count <- 0
    for (iter in seq_len(n_iter)) {
      prop_par <- curr_par + stats::rnorm(3, mean = 0, sd = scale_vec)
      if (prop_par[3] > 1e-6) {
        prop_loglik <- loglik_fn(prop_par)
        
        # Log prior (normal for a, b, 1/sigma for sigma)
        prior_curr <- stats::dnorm(curr_par[1], 0, 100, log = TRUE) +
                      stats::dnorm(curr_par[2], 0, 10000, log = TRUE) - log(curr_par[3])
        prior_prop <- stats::dnorm(prop_par[1], 0, 100, log = TRUE) +
                      stats::dnorm(prop_par[2], 0, 10000, log = TRUE) - log(prop_par[3])
        
        log_alpha <- (prop_loglik + prior_prop) - (curr_loglik + prior_curr)
        if (!is.na(log_alpha) && log(stats::runif(1)) < log_alpha) {
          curr_par <- prop_par
          curr_loglik <- prop_loglik
          acc_count <- acc_count + 1
        }
      }
      chain[iter, ] <- curr_par
    }
    
    post_chain <- chain[(burnin + 1):n_iter, , drop = FALSE]
    if (thin > 1) {
      post_chain <- post_chain[seq(1, nrow(post_chain), by = thin), , drop = FALSE]
    }
    
    par_mean <- colMeans(post_chain)
    par_sd <- apply(post_chain, 2, stats::sd)
    par_quant <- t(apply(post_chain, 2, stats::quantile, probs = c(0.025, 0.5, 0.975)))
    
    res <- list(
      par = par_mean,
      sd = par_sd,
      quantiles = par_quant,
      chain = post_chain,
      acceptance_rate = acc_count / n_iter,
      loglik = loglik_fn(par_mean),
      process = "wiener",
      model = model,
      method = "mcmc",
      stress_levels = stress_levels,
      thresholds = thresholds,
      tau_mat = tau_mat,
      units_data = units_data
    )
  }
  
  class(res) <- "ssadfit"
  return(res)
}

# Helper functions for data parsing and tau estimation
parse_ssad_data <- function(data) {
  if (is.data.frame(data)) {
    if (all(c("unit", "time", "degradation") %in% colnames(data))) {
      u_names <- unique(data$unit)
      units_list <- lapply(u_names, function(u) {
        sub_d <- data[data$unit == u, ]
        sub_d <- sub_d[order(sub_d$time), ]
        data.frame(time = sub_d$time, degradation = sub_d$degradation)
      })
      names(units_list) <- paste0("unit", u_names)
      return(list(units = units_list))
    } else {
      times <- data[[1]]
      u_cols <- colnames(data)[-1]
      units_list <- lapply(u_cols, function(col) {
        data.frame(time = times, degradation = data[[col]])
      })
      names(units_list) <- u_cols
      return(list(units = units_list))
    }
  } else {
    stop("data must be a data frame.")
  }
}

estimate_ssad_taus <- function(units_data, thresholds) {
  N <- length(units_data)
  K_sub <- length(thresholds)
  tau_mat <- matrix(0, nrow = N, ncol = K_sub)
  colnames(tau_mat) <- paste0("tau", seq_len(K_sub))
  
  for (i in seq_len(N)) {
    u_df <- units_data[[i]]
    t_vec <- u_df$time
    x_vec <- u_df$degradation
    
    for (k in seq_len(K_sub)) {
      om <- thresholds[k]
      idx <- which(x_vec >= om)
      if (length(idx) == 0) {
        # If not crossed, extrapolate based on last slope
        slope <- (max(x_vec) - min(x_vec)) / (max(t_vec) - min(t_vec))
        t_est <- max(t_vec) + (om - max(x_vec)) / max(slope, 1e-4)
      } else if (idx[1] == 1) {
        t_est <- t_vec[1]
      } else {
        # Linear interpolation
        j <- idx[1]
        t1 <- t_vec[j - 1]; t2 <- t_vec[j]
        x1 <- x_vec[j - 1]; x2 <- x_vec[j]
        t_est <- t1 + (om - x1) * (t2 - t1) / max(x2 - x1, 1e-6)
      }
      tau_mat[i, k] <- t_est
    }
  }
  return(tau_mat)
}

#' Geweke's MCMC Convergence Diagnostic
#'
#' Computes Geweke's convergence diagnostic for MCMC chains by comparing the sample means of
#' early and late parts of the Markov chain.
#'
#' @param chain A numeric matrix or data frame where columns correspond to parameters and rows to iterations,
#'   or a numeric vector for a single parameter.
#' @param frac1 Fraction of the chain to take from the beginning (default: 0.1).
#' @param frac2 Fraction of the chain to take from the end (default: 0.5).
#'
#' @return A data frame containing:
#'   \item{parameter}{Name of the parameter.}
#'   \item{mean_first}{Mean of the initial fraction of iterations.}
#'   \item{mean_last}{Mean of the final fraction of iterations.}
#'   \item{z_score}{Geweke z-statistic.}
#'   \item{p_value}{Two-sided p-value for equality of means.}
#'   \item{converged}{Logical indicating whether convergence is supported at alpha = 0.05 level.}
#'
#' @details
#' Geweke's convergence diagnostic tests for equality of the mean of early simulations
#' (default first 10\%) and the mean of later iterations (default last 50\%). Under the null hypothesis
#' of convergence, the test statistic follows a standard normal distribution.
#'
#' @references
#' Geweke, J. (1992). Evaluating the accuracy of sampling-based approaches to the calculation of posterior moments.
#' \emph{Bayesian Statistics}, 4, 169-193.
#'
#' Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
#' \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#'
#' @export
#' @examples
#' set.seed(123)
#' sample_chain <- matrix(rnorm(1000), ncol = 2, dimnames = list(NULL, c("param1", "param2")))
#' diag_res <- geweke_diag(sample_chain)
#' print(diag_res)
geweke_diag <- function(chain, frac1 = 0.1, frac2 = 0.5) {
  if (is.vector(chain)) {
    pnames <- deparse(substitute(chain))
    if (length(pnames) == 0 || nchar(pnames) == 0) pnames <- "param"
    chain <- matrix(chain, ncol = 1, dimnames = list(NULL, pnames))
  } else {
    chain <- as.matrix(chain)
  }
  
  if (is.null(colnames(chain))) {
    colnames(chain) <- paste0("param", seq_len(ncol(chain)))
  }
  
  n <- nrow(chain)
  n1 <- floor(frac1 * n)
  n2 <- floor(frac2 * n)
  
  if (n1 < 2 || n2 < 2 || (n - n2 + 1) <= n1) {
    stop("Chain length is too short for the specified fractions.")
  }
  
  start2 <- n - n2 + 1
  
  spec_var <- function(x) {
    len <- length(x)
    v <- stats::var(x) / len
    # AR(1) spectral density at zero frequency estimation for robust variance of mean
    if (len > 10) {
      sd_val <- stats::sd(x)
      if (!is.na(sd_val) && sd_val > 1e-12) {
        r1 <- stats::cor(x[-1], x[-len])
        if (!is.na(r1) && abs(r1) < 0.99) {
          v <- v * (1 + r1) / (1 - r1)
        }
      }
    }
    return(max(v, 1e-12))
  }
  
  results <- list()
  for (j in seq_len(ncol(chain))) {
    x <- chain[, j]
    x1 <- x[1:n1]
    x2 <- x[start2:n]
    
    m1 <- mean(x1)
    m2 <- mean(x2)
    
    v1 <- spec_var(x1)
    v2 <- spec_var(x2)
    
    z <- (m1 - m2) / sqrt(v1 + v2)
    p_val <- 2 * (1 - stats::pnorm(abs(z)))
    
    results[[j]] <- data.frame(
      parameter = colnames(chain)[j],
      mean_first = m1,
      mean_last = m2,
      z_score = z,
      p_value = p_val,
      converged = (p_val > 0.05),
      stringsAsFactors = FALSE
    )
  }
  
  res_df <- do.call(rbind, results)
  rownames(res_df) <- NULL
  return(res_df)
}

test_that("Datasets are properly loaded", {
  data(wiener_arrhenius)
  data(wiener_power)
  data(gamma_arrhenius)
  data(gamma_power)
  
  expect_equal(nrow(wiener_arrhenius), 30)
  expect_equal(ncol(wiener_arrhenius), 6)
  expect_false(any(is.na(wiener_arrhenius)))
  
  expect_equal(nrow(gamma_arrhenius), 30)
  expect_equal(ncol(gamma_arrhenius), 6)
  expect_false(any(is.na(gamma_arrhenius)))
})

test_that("Wiener process MLE and MCMC fitting works without NA/NaN", {
  data(wiener_arrhenius)
  
  fit_mle <- ssad_fit(
    data = wiener_arrhenius,
    process = "wiener",
    model = "arrhenius",
    stress_levels = c(45, 65, 85),
    thresholds = c(90, 160),
    method = "mle"
  )
  
  expect_s3_class(fit_mle, "ssadfit")
  expect_false(any(is.na(fit_mle$par)))
  expect_false(any(is.nan(fit_mle$par)))
  expect_true(fit_mle$loglik > -1e6)
  
  fit_mcmc <- ssad_fit(
    data = wiener_arrhenius,
    process = "wiener",
    model = "power",
    stress_levels = c(45, 65, 85),
    thresholds = c(90, 160),
    method = "mcmc",
    n_iter = 200,
    burnin = 50
  )
  
  expect_s3_class(fit_mcmc, "ssadfit")
  expect_false(any(is.na(fit_mcmc$par)))
  expect_false(any(is.nan(fit_mcmc$par)))
})

test_that("Gamma process MLE and MCMC fitting works without NA/NaN", {
  data(gamma_arrhenius)
  
  fit_mle <- ssad_fit(
    data = gamma_arrhenius,
    process = "gamma",
    model = "arrhenius",
    stress_levels = c(45, 65, 85),
    thresholds = c(90, 160),
    method = "mle"
  )
  
  expect_s3_class(fit_mle, "ssadfit")
  expect_false(any(is.na(fit_mle$par)))
  expect_false(any(is.nan(fit_mle$par)))
  
  fit_mcmc <- ssad_fit(
    data = gamma_arrhenius,
    process = "gamma",
    model = "power",
    stress_levels = c(45, 65, 85),
    thresholds = c(90, 160),
    method = "mcmc",
    n_iter = 200,
    burnin = 50
  )
  
  expect_s3_class(fit_mcmc, "ssadfit")
  expect_false(any(is.na(fit_mcmc$par)))
  expect_false(any(is.nan(fit_mcmc$par)))
})

test_that("Geweke diagnostic works properly", {
  set.seed(42)
  samp <- matrix(rnorm(500), ncol = 2, dimnames = list(NULL, c("a", "b")))
  g_res <- geweke_diag(samp)
  
  expect_equal(nrow(g_res), 2)
  expect_false(any(is.na(g_res$z_score)))
  expect_false(any(is.nan(g_res$p_value)))
})

test_that("Reliability prediction works properly", {
  rel <- ssad_reliability(
    t = seq(100, 1000, by = 100),
    process = "wiener",
    model = "arrhenius",
    params = c(a = 5.4, b = -2500, sigma = 0.1),
    S0 = 25,
    omega_F = 200
  )
  
  expect_s3_class(rel, "ssad_reliability")
  expect_equal(length(rel$reliability), 10)
  expect_true(rel$mttf > 0)
  expect_false(any(is.na(rel$reliability)))
})

test_that("Simulation function produces clean data", {
  set.seed(42)
  sim_d <- ssad_simulate(
    n_units = 3,
    times = seq(50, 500, by = 50),
    stress_levels = c(45, 65),
    thresholds = c(90),
    process = "gamma",
    model = "arrhenius",
    params = c(a = 6.7, b = -2500, u = 4.0)
  )
  
  expect_equal(nrow(sim_d), 10)
  expect_equal(ncol(sim_d), 4)
  expect_false(any(is.na(sim_d)))
})

#' Print Method for Step-Stress Accelerated Degradation Fit
#'
#' Prints a concise summary of the fitted step-stress accelerated degradation model.
#'
#' @param x An object of class \code{"ssadfit"}.
#' @param ... Additional arguments passed to print.
#'
#' @return Invisibly returns the input object \code{x}.
#' @export
#' @examples
#' data(wiener_arrhenius)
#' fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
#' print(fit)
print.ssadfit <- function(x, ...) {
  cat("\n=======================================================\n")
  cat("  Step-Stress Accelerated Degradation Model Fit\n")
  cat("=======================================================\n")
  cat("Process Model :", toupper(substr(x$process, 1, 1)), substring(x$process, 2), "Process\n")
  cat("Acceleration  :", toupper(substr(x$model, 1, 1)), substring(x$model, 2), "Model\n")
  cat("Method        :", toupper(x$method), "\n\n")
  
  cat("Estimated Parameters:\n")
  print(x$par)
  cat("\nLog-Likelihood:", format(x$loglik, digits = 6), "\n")
  cat("=======================================================\n\n")
  invisible(x)
}

#' Summary Method for Step-Stress Accelerated Degradation Fit
#'
#' Produces a detailed statistical summary of a fitted step-stress degradation model,
#' including parameter estimates, standard errors / MCMC quantiles, information criteria (AIC, BIC),
#' and MCMC convergence diagnostics (if fitted via MCMC).
#'
#' @param object An object of class \code{"ssadfit"}.
#' @param ... Additional arguments passed to summary.
#'
#' @return Invisibly returns the input object \code{object}.
#' @export
#' @examples
#' data(wiener_arrhenius)
#' fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
#' summary(fit)
summary.ssadfit <- function(object, ...) {
  cat("\n=======================================================\n")
  cat("  Detailed Summary: SSADT Model Fit\n")
  cat("=======================================================\n")
  cat("Process       :", object$process, "\n")
  cat("Acceleration  :", object$model, "\n")
  cat("Method        :", object$method, "\n\n")
  
  k_params <- length(object$par)
  n_obs <- sum(sapply(object$units_data, nrow))
  aic_val <- 2 * k_params - 2 * object$loglik
  bic_val <- log(n_obs) * k_params - 2 * object$loglik
  
  cat("Model Selection Criteria:\n")
  cat("  Log-Likelihood :", format(object$loglik, digits = 6), "\n")
  cat("  AIC            :", format(aic_val, digits = 6), "\n")
  cat("  BIC            :", format(bic_val, digits = 6), "\n\n")
  
  if (object$method == "mle") {
    cat("Maximum Likelihood Estimates:\n")
    df_est <- data.frame(
      Estimate = object$par,
      Std.Error = object$se
    )
    print(df_est)
  } else { # MCMC
    cat("Bayesian MCMC Posterior Statistics:\n")
    df_est <- data.frame(
      Mean = object$par,
      Std.Dev = object$sd,
      MC.Error = object$sd / sqrt(nrow(object$chain)),
      object$quantiles
    )
    print(df_est)
    
    cat("\nMCMC Acceptance Rate:", format(object$acceptance_rate * 100, digits = 4), "%\n")
    cat("\nGeweke's MCMC Convergence Diagnostics:\n")
    g_diag <- geweke_diag(object$chain)
    print(g_diag)
  }
  cat("=======================================================\n\n")
  invisible(object)
}

#' Plot Diagnostics and Degradation Paths for SSADT Models
#'
#' Generates diagnostic plots for a fitted step-stress degradation model, including observed vs. fitted
#' degradation paths and MCMC trace/posterior density plots (if fitted via MCMC).
#'
#' @param x An object of class \code{"ssadfit"}.
#' @param ... Additional graphical parameters.
#'
#' @return Invisibly returns the input object \code{x}. Side effect: displays diagnostic plots.
#' @importFrom grDevices rainbow
#' @importFrom graphics par plot lines points abline
#' @importFrom utils head
#' @export
#' @examples
#' data(wiener_arrhenius)
#' fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
#' plot(fit)
plot.ssadfit <- function(x, ...) {
  old_par <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(old_par))
  
  if (x$method == "mcmc" && !is.null(x$chain)) {
    graphics::par(mfrow = c(2, 2), mar = c(4, 4, 2, 1))
    
    # Subplot 1: Observed degradation paths
    units_data <- x$units_data
    t_max <- max(sapply(units_data, function(u) max(u$time)))
    x_max <- max(sapply(units_data, function(u) max(u$degradation)))
    
    graphics::plot(NULL, xlim = c(0, t_max), ylim = c(0, x_max * 1.1),
                   xlab = "Time (hours)", ylab = "Degradation Value",
                   main = paste("Degradation Paths (", x$process, ")", sep = ""))
    
    colors <- grDevices::rainbow(length(units_data))
    for (i in seq_along(units_data)) {
      u_df <- units_data[[i]]
      graphics::lines(u_df$time, u_df$degradation, col = colors[i], lwd = 2)
      graphics::points(u_df$time, u_df$degradation, col = colors[i], pch = 16, cex = 0.6)
    }
    for (om in x$thresholds) {
      graphics::abline(h = om, lty = 2, col = "gray40")
    }
    
    # Subplots 2-4: MCMC trace plots for parameters
    chain <- x$chain
    for (j in seq_len(ncol(chain))) {
      pname <- colnames(chain)[j]
      graphics::plot(chain[, j], type = "l", col = "blue",
                     xlab = "Iteration", ylab = pname,
                     main = paste("Trace Plot:", pname))
    }
  } else {
    graphics::par(mfrow = c(1, 1), mar = c(4, 4, 3, 1))
    units_data <- x$units_data
    t_max <- max(sapply(units_data, function(u) max(u$time)))
    x_max <- max(sapply(units_data, function(u) max(u$degradation)))
    
    graphics::plot(NULL, xlim = c(0, t_max), ylim = c(0, x_max * 1.1),
                   xlab = "Time (hours)", ylab = "Degradation Value",
                   main = paste("Step-Stress Accelerated Degradation Paths (", toupper(x$process), ")", sep = ""))
    
    colors <- grDevices::rainbow(length(units_data))
    for (i in seq_along(units_data)) {
      u_df <- units_data[[i]]
      graphics::lines(u_df$time, u_df$degradation, col = colors[i], lwd = 2)
      graphics::points(u_df$time, u_df$degradation, col = colors[i], pch = 16, cex = 0.7)
    }
    for (om in x$thresholds) {
      graphics::abline(h = om, lty = 2, col = "gray40")
    }
  }
  
  invisible(x)
}

#' Predict Lifetime Distribution and Reliability for SSADT Fit
#'
#' Predicts the reliability function R(t), cumulative failure distribution F(t), and MTTF at normal operating
#' stress conditions S0 based on a fitted \code{"ssadfit"} model.
#'
#' @param object An object of class \code{"ssadfit"}.
#' @param t Numeric vector of time points for reliability evaluation (default: \code{seq(100, 10000, by = 100)}).
#' @param S0 Normal use-stress level (default: 25).
#' @param omega_F Failure threshold for degradation (default: 200).
#' @param ... Additional prediction arguments.
#'
#' @return An object of class \code{"ssad_reliability"} containing time, reliability R(t), failure CDF F(t), and MTTF.
#' @export
#' @examples
#' data(wiener_arrhenius)
#' fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
#' pred <- predict(fit, t = seq(100, 5000, by = 100), S0 = 25, omega_F = 200)
#' print(pred$mttf)
predict.ssadfit <- function(object, t = seq(100, 10000, by = 100), S0 = 25, omega_F = 200, ...) {
  rel <- ssad_reliability(
    t = t,
    process = object$process,
    model = object$model,
    params = object$par,
    S0 = S0,
    omega_F = omega_F
  )
  return(rel)
}

#' Fit Multiple-Steps Step-Stress Accelerated Degradation Model
#'
#' Master fitting function for step-stress accelerated degradation testing (SSADT) models using
#' Wiener process or Gamma process degradation models with Arrhenius or Power acceleration relationships.
#'
#' @param data A data frame containing inspection times in the first column and unit degradation measurements in subsequent columns,
#'   or long-format data with columns \code{unit}, \code{time}, \code{degradation}.
#' @param process Character string specifying degradation process model: \code{"wiener"} or \code{"gamma"}.
#' @param model Acceleration model: \code{"arrhenius"} or \code{"power"}.
#' @param stress_levels Vector of stress levels S = (S1, S2, ..., SK). Default: \code{c(45, 65, 85)}.
#' @param thresholds Vector of degradation thresholds (omega_1, ..., omega_K-1) at which stress levels are elevated. Default: \code{c(90, 160)}.
#' @param method Estimation method: \code{"mle"} or \code{"mcmc"}. Default: \code{"mle"}.
#' @param init Initial parameter vector.
#' @param n_iter Number of MCMC iterations (default: 5000).
#' @param burnin Number of burn-in iterations (default: 1000).
#' @param thin Thinning interval for MCMC (default: 1).
#'
#' @return An S3 object of class \code{"ssadfit"} containing model fit results, parameter estimates, log-likelihood, and diagnostics.
#'
#' @references
#' Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
#' \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#'
#' @export
#' @examples
#' data(wiener_arrhenius)
#' fit <- ssad_fit(
#'   data = wiener_arrhenius,
#'   process = "wiener",
#'   model = "arrhenius",
#'   stress_levels = c(45, 65, 85),
#'   thresholds = c(90, 160),
#'   method = "mle"
#' )
#' print(fit)
#' summary(fit)
ssad_fit <- function(data, process = c("wiener", "gamma"), model = c("arrhenius", "power"),
                     stress_levels = c(45, 65, 85), thresholds = c(90, 160),
                     method = c("mle", "mcmc"), init = NULL, n_iter = 5000,
                     burnin = 1000, thin = 1) {
  process <- match.arg(process)
  model <- match.arg(model)
  method <- match.arg(method)
  
  if (process == "wiener") {
    res <- ssad_wiener(
      data = data,
      stress_levels = stress_levels,
      thresholds = thresholds,
      model = model,
      method = method,
      init = init,
      n_iter = n_iter,
      burnin = burnin,
      thin = thin
    )
  } else {
    res <- ssad_gamma(
      data = data,
      stress_levels = stress_levels,
      thresholds = thresholds,
      model = model,
      method = method,
      init = init,
      n_iter = n_iter,
      burnin = burnin,
      thin = thin
    )
  }
  
  return(res)
}

#' Simulate Step-Stress Accelerated Degradation Data
#'
#' Generates simulated degradation path data for N test units subject to a K-steps step-stress
#' accelerated degradation test (SSADT) under Wiener or Gamma stochastic process models.
#'
#' @param n_units Number of test units (e.g. 5).
#' @param times Vector of inspection time points (e.g. \code{seq(72, 2160, by = 72)}).
#' @param stress_levels Vector of stress levels S = (S1, S2, ..., SK).
#' @param thresholds Vector of degradation thresholds (omega_1, ..., omega_K-1) at which stress is elevated.
#' @param process Character string specifying \code{"wiener"} or \code{"gamma"}.
#' @param model Acceleration model: \code{"arrhenius"} or \code{"power"}.
#' @param params Model parameters: \code{c(a, b, sigma)} for Wiener, or \code{c(a, b, u)} for Gamma.
#'
#' @return A data frame with columns \code{time}, \code{unit1}, \code{unit2}, ..., \code{unitN}.
#'
#' @references
#' Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
#' \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#'
#' @export
#' @examples
#' set.seed(42)
#' sim_data <- ssad_simulate(
#'   n_units = 5,
#'   times = seq(72, 2160, by = 72),
#'   stress_levels = c(45, 65, 85),
#'   thresholds = c(90, 160),
#'   process = "wiener",
#'   model = "arrhenius",
#'   params = c(a = 5.3669, b = -2546.7, sigma = 0.1)
#' )
#' head(sim_data)
ssad_simulate <- function(n_units = 5, times = seq(72, 2160, by = 72),
                          stress_levels = c(45, 65, 85), thresholds = c(90, 160),
                          process = c("wiener", "gamma"), model = c("arrhenius", "power"),
                          params) {
  process <- match.arg(process)
  if (is.character(model)) {
    model <- match.arg(model)
  }
  
  eval_acc <- function(S, m_type, par) {
    if (is.function(m_type)) {
      return(m_type(S, par))
    }
    a <- par[1]
    b <- par[2]
    if (m_type == "arrhenius") {
      return(exp(a + b / (273 + S)))
    } else if (m_type == "power") {
      return(a * (S^b))
    } else {
      stop("Unsupported acceleration model.")
    }
  }
  
  K <- length(stress_levels)
  if (length(thresholds) != K - 1) {
    stop("Length of thresholds must be length(stress_levels) - 1.")
  }
  
  M <- length(times)
  dt_vec <- c(times[1], diff(times))
  
  res_mat <- matrix(0, nrow = M, ncol = n_units)
  
  for (i in seq_len(n_units)) {
    x_curr <- 0
    t_curr <- 0
    k_curr <- 1
    
    for (j in seq_len(M)) {
      dt <- dt_vec[j]
      
      # Check current stress level based on x_curr
      while (k_curr < K && x_curr >= thresholds[k_curr]) {
        k_curr <- k_curr + 1
      }
      
      rate_val <- eval_acc(stress_levels[k_curr], model, params[1:2])
      
      if (process == "wiener") {
        sigma <- params[3]
        dx <- stats::rnorm(1, mean = rate_val * dt, sd = sigma * sqrt(dt))
      } else { # gamma
        u <- params[3]
        shape_val <- rate_val * dt
        dx <- stats::rgamma(1, shape = shape_val, rate = u)
      }
      
      x_curr <- x_curr + dx
      res_mat[j, i] <- x_curr
    }
  }
  
  df_res <- data.frame(time = times, res_mat)
  colnames(df_res) <- c("time", paste0("unit", seq_len(n_units)))
  return(df_res)
}

#' Wiener-Arrhenius Step-Stress Accelerated Degradation Data
#'
#' Simulated degradation dataset generated using a Wiener process with an Arrhenius
#' acceleration model under a 3-step step-stress accelerated degradation test (SSADT).
#' Originally published in Table 2 of Pan and Balakrishnan (2010).
#'
#' @format A data frame with 30 rows and 6 variables:
#' \describe{
#'   \item{time}{Inspection time in hours (72 to 2160 hours).}
#'   \item{unit1}{Degradation measurements for test unit 1.}
#'   \item{unit2}{Degradation measurements for test unit 2.}
#'   \item{unit3}{Degradation measurements for test unit 3.}
#'   \item{unit4}{Degradation measurements for test unit 4.}
#'   \item{unit5}{Degradation measurements for test unit 5.}
#' }
#' @source Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes. \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#' @docType data
#' @keywords datasets
#' @name wiener_arrhenius
#' @usage data(wiener_arrhenius)
#' @examples
#' data(wiener_arrhenius)
#' head(wiener_arrhenius)
"wiener_arrhenius"

#' Wiener-Power Step-Stress Accelerated Degradation Data
#'
#' Simulated degradation dataset generated using a Wiener process with a Power
#' acceleration model under a 3-step step-stress accelerated degradation test (SSADT).
#' Originally published in Table 3 of Pan and Balakrishnan (2010).
#'
#' @format A data frame with 30 rows and 6 variables:
#' \describe{
#'   \item{time}{Inspection time in hours (72 to 2160 hours).}
#'   \item{unit1}{Degradation measurements for test unit 1.}
#'   \item{unit2}{Degradation measurements for test unit 2.}
#'   \item{unit3}{Degradation measurements for test unit 3.}
#'   \item{unit4}{Degradation measurements for test unit 4.}
#'   \item{unit5}{Degradation measurements for test unit 5.}
#' }
#' @source Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes. \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#' @docType data
#' @keywords datasets
#' @name wiener_power
#' @usage data(wiener_power)
#' @examples
#' data(wiener_power)
#' head(wiener_power)
"wiener_power"

#' Gamma-Arrhenius Step-Stress Accelerated Degradation Data
#'
#' Simulated degradation dataset generated using a Gamma process with an Arrhenius
#' acceleration model under a 3-step step-stress accelerated degradation test (SSADT).
#' Originally published in Table 4 of Pan and Balakrishnan (2010).
#'
#' @format A data frame with 30 rows and 6 variables:
#' \describe{
#'   \item{time}{Inspection time in hours (72 to 2160 hours).}
#'   \item{unit1}{Degradation measurements for test unit 1.}
#'   \item{unit2}{Degradation measurements for test unit 2.}
#'   \item{unit3}{Degradation measurements for test unit 3.}
#'   \item{unit4}{Degradation measurements for test unit 4.}
#'   \item{unit5}{Degradation measurements for test unit 5.}
#' }
#' @source Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes. \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#' @docType data
#' @keywords datasets
#' @name gamma_arrhenius
#' @usage data(gamma_arrhenius)
#' @examples
#' data(gamma_arrhenius)
#' head(gamma_arrhenius)
"gamma_arrhenius"

#' Gamma-Power Step-Stress Accelerated Degradation Data
#'
#' Simulated degradation dataset generated using a Gamma process with a Power
#' acceleration model under a 3-step step-stress accelerated degradation test (SSADT).
#' Originally published in Table 5 of Pan and Balakrishnan (2010).
#'
#' @format A data frame with 30 rows and 6 variables:
#' \describe{
#'   \item{time}{Inspection time in hours (72 to 2160 hours).}
#'   \item{unit1}{Degradation measurements for test unit 1.}
#'   \item{unit2}{Degradation measurements for test unit 2.}
#'   \item{unit3}{Degradation measurements for test unit 3.}
#'   \item{unit4}{Degradation measurements for test unit 4.}
#'   \item{unit5}{Degradation measurements for test unit 5.}
#' }
#' @source Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes. \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#' @docType data
#' @keywords datasets
#' @name gamma_power
#' @usage data(gamma_power)
#' @examples
#' data(gamma_power)
#' head(gamma_power)
"gamma_power"

#' Gamma Process Step-Stress Accelerated Degradation Model Estimation
#'
#' Fits a Gamma process step-stress accelerated degradation model using Maximum Likelihood Estimation (MLE)
#' or Bayesian Markov Chain Monte Carlo (MCMC) based on the Birnbaum-Saunders approximation for stress elevation times
#' as described in Pan and Balakrishnan (2010).
#'
#' @param data A data frame containing inspection times in the first column and unit degradation measurements in subsequent columns,
#'   or a long-format data frame with columns \code{unit}, \code{time}, \code{degradation}.
#' @param stress_levels Vector of stress levels S = (S1, S2, ..., SK).
#' @param thresholds Vector of degradation thresholds (omega_1, ..., omega_K-1) at which stress levels are elevated.
#' @param model Acceleration model: \code{"arrhenius"} or \code{"power"}.
#' @param method Estimation method: \code{"mle"} or \code{"mcmc"}.
#' @param init Initial parameter values \code{c(a, b, u)}.
#' @param n_iter Number of MCMC iterations (default: 5000).
#' @param burnin Number of burn-in iterations (default: 1000).
#' @param thin Thinning interval for MCMC (default: 1).
#'
#' @return An object of class \code{"ssadfit"} containing fit results, parameter estimates, log-likelihood, MCMC samples (if applicable), and metadata.
#'
#' @references
#' Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
#' \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#'
#' @export
#' @examples
#' data(gamma_arrhenius)
#' fit_g <- ssad_gamma(
#'   data = gamma_arrhenius,
#'   stress_levels = c(45, 65, 85),
#'   thresholds = c(90, 160),
#'   model = "arrhenius",
#'   method = "mle"
#' )
#' print(fit_g)
ssad_gamma <- function(data, stress_levels = c(45, 65, 85), thresholds = c(90, 160),
                       model = c("arrhenius", "power"), method = c("mle", "mcmc"),
                       init = NULL, n_iter = 5000, burnin = 1000, thin = 1) {
  model <- match.arg(model)
  method <- match.arg(method)
  
  parsed_data <- parse_ssad_data(data)
  units_data <- parsed_data$units
  
  K <- length(stress_levels)
  if (length(thresholds) != K - 1) {
    stop("Length of thresholds must equal length(stress_levels) - 1.")
  }
  
  tau_mat <- estimate_ssad_taus(units_data, thresholds)
  
  eval_acc <- function(S, m_type, par) {
    if (is.function(m_type)) return(m_type(S, par))
    a <- par[1]; b <- par[2]
    if (m_type == "arrhenius") return(exp(a + b / (273 + S)))
    if (m_type == "power") return(a * (S^b))
    stop("Unsupported acceleration model.")
  }
  
  loglik_fn <- function(par) {
    a <- par[1]; b <- par[2]; u <- par[3]
    if (is.na(u) || u <= 1e-6) return(-1e12)
    
    v_k <- sapply(stress_levels, function(s) eval_acc(s, model, c(a, b)))
    if (any(is.na(v_k)) || any(v_k <= 1e-8)) return(-1e12)
    
    logL <- 0
    N <- length(units_data)
    
    # 1. Stress elevation time likelihood g(tau_i) via Birnbaum-Saunders approximation
    d_omega <- c(thresholds[1], diff(thresholds))
    for (i in seq_len(N)) {
      tau_i <- tau_mat[i, ]
      d_tau <- c(tau_i[1], diff(tau_i))
      for (k in seq_len(K - 1)) {
        dtk <- max(d_tau[k], 1e-6)
        dok <- d_omega[k]
        vk <- v_k[k]
        
        ratio <- (vk * dtk) / (u * dok)
        if (is.na(ratio) || ratio <= 1e-10) return(-1e12)
        
        term1 <- log(vk / (2 * sqrt(2 * pi * u * dok)))
        term2 <- log((ratio)^(-0.5) + (ratio)^(-1.5))
        term3 <- -(u * dok / 2) * (ratio - 2 + 1 / ratio)
        
        logL <- logL + term1 + term2 + term3
      }
    }
    
    # 2. Degradation increment likelihood f(dx_{i,j}) via Gamma process
    for (i in seq_len(N)) {
      u_df <- units_data[[i]]
      t_vec <- u_df$time
      x_vec <- u_df$degradation
      tau_i <- c(0, tau_mat[i, ], max(t_vec) * 10)
      
      M <- length(t_vec)
      for (j in seq_len(M - 1)) {
        t1 <- t_vec[j]; t2 <- t_vec[j + 1]
        dx <- max(x_vec[j + 1] - x_vec[j], 1e-8)
        
        # Calculate shape parameter upsilon_ij based on stress level overlaps
        upsilon_ij <- 0
        for (k in seq_len(K)) {
          t_start_k <- tau_i[k]
          t_end_k <- tau_i[k + 1]
          overlap <- max(0, min(t2, t_end_k) - max(t1, t_start_k))
          if (overlap > 0) {
            upsilon_ij <- upsilon_ij + v_k[k] * overlap
          }
        }
        
        upsilon_ij <- max(upsilon_ij, 1e-6)
        
        term1 <- upsilon_ij * log(u) - lgamma(upsilon_ij)
        term2 <- (upsilon_ij - 1) * log(dx) - u * dx
        
        logL <- logL + term1 + term2
      }
    }
    
    if (is.na(logL) || is.nan(logL)) return(-1e12)
    return(logL)
  }
  
  if (is.null(init)) {
    if (model == "arrhenius") {
      init <- c(a = 6.7, b = -2500, u = 4.0)
    } else {
      init <- c(a = 3e-3, b = 1.2, u = 4.0)
    }
  }
  
  if (method == "mle") {
    neg_loglik <- function(p) -loglik_fn(p)
    opt <- stats::optim(init, neg_loglik, method = "Nelder-Mead", control = list(maxit = 2000))
    opt <- stats::optim(opt$par, neg_loglik, method = "BFGS", hessian = TRUE, control = list(maxit = 2000))
    
    par_mle <- opt$par
    names(par_mle) <- c("a", "b", "u")
    
    se_mle <- rep(NA, 3)
    if (!is.null(opt$hessian) && !any(is.na(opt$hessian))) {
      cov_mat <- tryCatch(solve(opt$hessian), error = function(e) NULL)
      if (!is.null(cov_mat)) {
        se_mle <- sqrt(pmax(diag(cov_mat), 0))
      }
    }
    names(se_mle) <- c("a", "b", "u")
    
    res <- list(
      par = par_mle,
      se = se_mle,
      loglik = -opt$value,
      process = "gamma",
      model = model,
      method = "mle",
      convergence = opt$convergence,
      stress_levels = stress_levels,
      thresholds = thresholds,
      tau_mat = tau_mat,
      units_data = units_data
    )
  } else { # MCMC
    chain <- matrix(0, nrow = n_iter, ncol = 3)
    colnames(chain) <- c("a", "b", "u")
    curr_par <- init
    curr_loglik <- loglik_fn(curr_par)
    
    scale_vec <- abs(init) * 0.05
    scale_vec[scale_vec == 0] <- 0.01
    
    acc_count <- 0
    for (iter in seq_len(n_iter)) {
      prop_par <- curr_par + stats::rnorm(3, mean = 0, sd = scale_vec)
      if (prop_par[3] > 1e-6) {
        prop_loglik <- loglik_fn(prop_par)
        
        prior_curr <- stats::dnorm(curr_par[1], 0, 100, log = TRUE) +
                      stats::dnorm(curr_par[2], 0, 10000, log = TRUE) - log(curr_par[3])
        prior_prop <- stats::dnorm(prop_par[1], 0, 100, log = TRUE) +
                      stats::dnorm(prop_par[2], 0, 10000, log = TRUE) - log(prop_par[3])
        
        log_alpha <- (prop_loglik + prior_prop) - (curr_loglik + prior_curr)
        if (!is.na(log_alpha) && log(stats::runif(1)) < log_alpha) {
          curr_par <- prop_par
          curr_loglik <- prop_loglik
          acc_count <- acc_count + 1
        }
      }
      chain[iter, ] <- curr_par
    }
    
    post_chain <- chain[(burnin + 1):n_iter, , drop = FALSE]
    if (thin > 1) {
      post_chain <- post_chain[seq(1, nrow(post_chain), by = thin), , drop = FALSE]
    }
    
    par_mean <- colMeans(post_chain)
    par_sd <- apply(post_chain, 2, stats::sd)
    par_quant <- t(apply(post_chain, 2, stats::quantile, probs = c(0.025, 0.5, 0.975)))
    
    res <- list(
      par = par_mean,
      sd = par_sd,
      quantiles = par_quant,
      chain = post_chain,
      acceptance_rate = acc_count / n_iter,
      loglik = loglik_fn(par_mean),
      process = "gamma",
      model = model,
      method = "mcmc",
      stress_levels = stress_levels,
      thresholds = thresholds,
      tau_mat = tau_mat,
      units_data = units_data
    )
  }
  
  class(res) <- "ssadfit"
  return(res)
}

#' Reliability and Lifetime Analysis for Step-Stress Accelerated Degradation Models
#'
#' Computes the reliability function R(t), cumulative failure distribution F(t), and Mean Time To Failure (MTTF)
#' at use stress level S0 and failure threshold omega_F based on fitted Wiener or Gamma process SSADT parameters.
#'
#' @param t Numeric vector of time points at which to evaluate reliability.
#' @param process Character string specifying the process type: \code{"wiener"} or \code{"gamma"}.
#' @param model Character string or function for acceleration model (\code{"arrhenius"} or \code{"power"}).
#' @param params Vector of estimated model parameters: \code{c(a, b, sigma)} for Wiener, or \code{c(a, b, u)} for Gamma.
#' @param S0 Normal use-stress level.
#' @param omega_F Failure threshold for degradation.
#'
#' @return A list of class \code{"ssad_reliability"} containing:
#'   \item{time}{Vector of time points.}
#'   \item{reliability}{Vector of reliability values R(t).}
#'   \item{cdf}{Vector of cumulative distribution function values F(t).}
#'   \item{mttf}{Mean Time To Failure under use stress S0.}
#'   \item{process}{Degradation process type.}
#'   \item{S0}{Use stress level.}
#'   \item{omega_F}{Failure threshold.}
#'
#' @references
#' Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
#' \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
#'
#' @export
#' @examples
#' # Wiener-Arrhenius model example
#' rel_wiener <- ssad_reliability(
#'   t = seq(100, 10000, by = 100),
#'   process = "wiener",
#'   model = "arrhenius",
#'   params = c(a = 5.4188, b = -2564.3, sigma = 0.0894),
#'   S0 = 25,
#'   omega_F = 200
#' )
#' print(rel_wiener$mttf)
ssad_reliability <- function(t, process = c("wiener", "gamma"), model = c("arrhenius", "power"),
                             params, S0, omega_F) {
  process <- match.arg(process)
  if (is.character(model)) {
    model <- match.arg(model)
  }
  
  eval_acc <- function(S, m_type, par) {
    if (is.function(m_type)) {
      return(m_type(S, par))
    }
    a <- par[1]
    b <- par[2]
    if (m_type == "arrhenius") {
      return(exp(a + b / (273 + S)))
    } else if (m_type == "power") {
      return(a * (S^b))
    } else {
      stop("Unsupported acceleration model.")
    }
  }
  
  if (process == "wiener") {
    beta0 <- eval_acc(S0, model, params[1:2])
    sigma <- params[3]
    mu0 <- omega_F / beta0
    lambda0 <- (omega_F^2) / (sigma^2)
    
    # Cumulative failure distribution F(t) for Inverse Gaussian
    t_clean <- pmax(t, 1e-8)
    z1 <- sqrt(lambda0 / t_clean) * (t_clean / mu0 - 1)
    z2 <- -sqrt(lambda0 / t_clean) * (t_clean / mu0 + 1)
    
    exp_factor <- min(2 * lambda0 / mu0, 500) # prevent numeric overflow
    cdf_val <- stats::pnorm(z1) + exp(exp_factor) * stats::pnorm(z2)
    cdf_val <- pmin(pmax(cdf_val, 0), 1)
    rel_val <- 1 - cdf_val
    mttf_val <- mu0
    
  } else if (process == "gamma") {
    v0 <- eval_acc(S0, model, params[1:2])
    u <- params[3]
    
    alpha0 <- 1 / sqrt(u * omega_F)
    beta0_star <- (u * omega_F) / v0
    
    t_clean <- pmax(t, 1e-8)
    z <- (1 / alpha0) * (sqrt(t_clean / beta0_star) - sqrt(beta0_star / t_clean))
    cdf_val <- stats::pnorm(z)
    cdf_val <- pmin(pmax(cdf_val, 0), 1)
    rel_val <- 1 - cdf_val
    mttf_val <- beta0_star * (1 + (alpha0^2) / 2)
  }
  
  res <- list(
    time = t,
    reliability = rel_val,
    cdf = cdf_val,
    mttf = mttf_val,
    process = process,
    S0 = S0,
    omega_F = omega_F
  )
  class(res) <- "ssad_reliability"
  return(res)
}

---
title: "Multiple-Steps Step-Stress Accelerated Degradation Modeling"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Multiple-Steps Step-Stress Accelerated Degradation Modeling}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

The **`MultiStepSSAD`** package provides generalized statistical tools for modeling **Multiple-Steps Step-Stress Accelerated Degradation Testing (SSADT)** data, implementing the methods developed by **Pan and Balakrishnan (2010)**.

In step-stress accelerated degradation experiments, products are subjected to elevated stress levels ($S_1 < S_2 < \dots < S_K$). Unlike traditional step-stress tests where stress levels are elevated at pre-determined fixed time points for all units, Pan and Balakrishnan (2010) proposed elevating stress levels when an individual product's degradation crosses pre-specified degradation threshold values ($\omega_1, \dots, \omega_{K-1}$). Consequently, the stress transition times ($\tau_{i,k}$) vary randomly from product to product.

This package supports:
1. **Wiener Process Degradation Models**: Drift parameter changes with stress level; diffusion parameter $\sigma$ remains constant.
2. **Gamma Process Degradation Models**: Monotone degradation process with independent Gamma-distributed increments. Stress transition times are modeled using the **Birnbaum-Saunders** distribution approximation.
3. **Acceleration Models**: Arrhenius model $A(S) = \exp(a + b / (273 + S))$ and Power model $A(S) = a \cdot S^b$.
4. **Estimation Methods**: Maximum Likelihood Estimation (MLE) and Bayesian Markov Chain Monte Carlo (MCMC).
5. **Diagnostics & Lifetime Prediction**: Geweke's MCMC convergence diagnostic, Mean Time To Failure (MTTF), and reliability $R(t)$ evaluation under normal use stress $S_0$.

---

## 1. Package Datasets

The package includes the exact simulated datasets from Tables 2–5 of Pan and Balakrishnan (2010):

```{r datasets}
library(MultiStepSSAD)

# Load Wiener-Arrhenius dataset (Table 2)
data(wiener_arrhenius)
head(wiener_arrhenius)

# Load Gamma-Arrhenius dataset (Table 4)
data(gamma_arrhenius)
head(gamma_arrhenius)
```

---

## 2. Fitting Step-Stress Accelerated Degradation Models

### 2.1. Wiener Process Model with Arrhenius Acceleration

We fit a 3-step step-stress Wiener-Arrhenius model to `wiener_arrhenius` data using Maximum Likelihood Estimation (MLE):

```{r wiener_mle}
fit_w_mle <- ssad_fit(
  data = wiener_arrhenius,
  process = "wiener",
  model = "arrhenius",
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  method = "mle"
)

summary(fit_w_mle)
```

### 2.2. Gamma Process Model with Arrhenius Acceleration

Next, we fit a 3-step step-stress Gamma-Arrhenius model to `gamma_arrhenius` data using Bayesian MCMC:

```{r gamma_mcmc}
fit_g_mcmc <- ssad_fit(
  data = gamma_arrhenius,
  process = "gamma",
  model = "arrhenius",
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  method = "mcmc",
  n_iter = 1000,
  burnin = 200
)

summary(fit_g_mcmc)
```

---

## 3. MCMC Convergence Diagnostics

The package includes Geweke's MCMC convergence diagnostic (`geweke_diag()`):

```{r geweke_diag}
if (!is.null(fit_g_mcmc$chain)) {
  g_res <- geweke_diag(fit_g_mcmc$chain)
  print(g_res)
}
```

---

## 4. Lifetime Prediction and Reliability Analysis

Using the fitted parameters, we can predict the reliability function $R(t)$ and Mean Time To Failure (MTTF) under normal operating stress conditions ($S_0 = 25^\circ\text{C}$) and failure threshold $\omega_F = 200$:

```{r reliability_pred}
rel_pred <- predict(
  fit_w_mle,
  t = seq(100, 10000, by = 200),
  S0 = 25,
  omega_F = 200
)

cat("Predicted MTTF under use stress S0 = 25:", round(rel_pred$mttf, 2), "hours\n")
```

---

## 5. Simulating Step-Stress Degradation Data

We can generate simulated step-stress degradation datasets for power calculation and experimental planning using `ssad_simulate()`:

```{r simulation}
set.seed(123)
sim_df <- ssad_simulate(
  n_units = 5,
  times = seq(72, 2160, by = 72),
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  process = "wiener",
  model = "power",
  params = c(a = 7.39e-4, b = 1.2, sigma = 0.1)
)

head(sim_df)
```

---

## References

- Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes. *Communications in Statistics - Simulation and Computation*, 39(7), 1384–1402. [doi:10.1080/03610918.2010.496060](https://doi.org/10.1080/03610918.2010.496060)
- Park, C., & Padgett, W. J. (2005). Accelerated degradation models for failure based on geometric Brownian motion and gamma processes. *Lifetime Data Analysis*, 11(4), 511–527.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.ssadfit}
\alias{summary.ssadfit}
\title{Summary Method for Step-Stress Accelerated Degradation Fit}
\usage{
\method{summary}{ssadfit}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"ssadfit"}.}

\item{...}{Additional arguments passed to summary.}
}
\value{
Invisibly returns the input object \code{object}.
}
\description{
Produces a detailed statistical summary of a fitted step-stress degradation model,
including parameter estimates, standard errors / MCMC quantiles, information criteria (AIC, BIC),
and MCMC convergence diagnostics (if fitted via MCMC).
}
\examples{
data(wiener_arrhenius)
fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
summary(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{ssad_reliability}
\alias{ssad_reliability}
\title{Reliability and Lifetime Analysis for Step-Stress Accelerated Degradation Models}
\usage{
ssad_reliability(
  t,
  process = c("wiener", "gamma"),
  model = c("arrhenius", "power"),
  params,
  S0,
  omega_F
)
}
\arguments{
\item{t}{Numeric vector of time points at which to evaluate reliability.}

\item{process}{Character string specifying the process type: \code{"wiener"} or \code{"gamma"}.}

\item{model}{Character string or function for acceleration model (\code{"arrhenius"} or \code{"power"}).}

\item{params}{Vector of estimated model parameters: \code{c(a, b, sigma)} for Wiener, or \code{c(a, b, u)} for Gamma.}

\item{S0}{Normal use-stress level.}

\item{omega_F}{Failure threshold for degradation.}
}
\value{
A list of class \code{"ssad_reliability"} containing:
  \item{time}{Vector of time points.}
  \item{reliability}{Vector of reliability values R(t).}
  \item{cdf}{Vector of cumulative distribution function values F(t).}
  \item{mttf}{Mean Time To Failure under use stress S0.}
  \item{process}{Degradation process type.}
  \item{S0}{Use stress level.}
  \item{omega_F}{Failure threshold.}
}
\description{
Computes the reliability function R(t), cumulative failure distribution F(t), and Mean Time To Failure (MTTF)
at use stress level S0 and failure threshold omega_F based on fitted Wiener or Gamma process SSADT parameters.
}
\examples{
# Wiener-Arrhenius model example
rel_wiener <- ssad_reliability(
  t = seq(100, 10000, by = 100),
  process = "wiener",
  model = "arrhenius",
  params = c(a = 5.4188, b = -2564.3, sigma = 0.0894),
  S0 = 25,
  omega_F = 200
)
print(rel_wiener$mttf)
}
\references{
Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
\emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{predict.ssadfit}
\alias{predict.ssadfit}
\title{Predict Lifetime Distribution and Reliability for SSADT Fit}
\usage{
\method{predict}{ssadfit}(object, t = seq(100, 10000, by = 100), S0 = 25, omega_F = 200, ...)
}
\arguments{
\item{object}{An object of class \code{"ssadfit"}.}

\item{t}{Numeric vector of time points for reliability evaluation (default: \code{seq(100, 10000, by = 100)}).}

\item{S0}{Normal use-stress level (default: 25).}

\item{omega_F}{Failure threshold for degradation (default: 200).}

\item{...}{Additional prediction arguments.}
}
\value{
An object of class \code{"ssad_reliability"} containing time, reliability R(t), failure CDF F(t), and MTTF.
}
\description{
Predicts the reliability function R(t), cumulative failure distribution F(t), and MTTF at normal operating
stress conditions S0 based on a fitted \code{"ssadfit"} model.
}
\examples{
data(wiener_arrhenius)
fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
pred <- predict(fit, t = seq(100, 5000, by = 100), S0 = 25, omega_F = 200)
print(pred$mttf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{wiener_power}
\alias{wiener_power}
\title{Wiener-Power Step-Stress Accelerated Degradation Data}
\format{
A data frame with 30 rows and 6 variables:
\describe{
  \item{time}{Inspection time in hours (72 to 2160 hours).}
  \item{unit1}{Degradation measurements for test unit 1.}
  \item{unit2}{Degradation measurements for test unit 2.}
  \item{unit3}{Degradation measurements for test unit 3.}
  \item{unit4}{Degradation measurements for test unit 4.}
  \item{unit5}{Degradation measurements for test unit 5.}
}
}
\source{
Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes. \emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
}
\usage{
data(wiener_power)
}
\description{
Simulated degradation dataset generated using a Wiener process with a Power
acceleration model under a 3-step step-stress accelerated degradation test (SSADT).
Originally published in Table 3 of Pan and Balakrishnan (2010).
}
\examples{
data(wiener_power)
head(wiener_power)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{ssad_simulate}
\alias{ssad_simulate}
\title{Simulate Step-Stress Accelerated Degradation Data}
\usage{
ssad_simulate(
  n_units = 5,
  times = seq(72, 2160, by = 72),
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  process = c("wiener", "gamma"),
  model = c("arrhenius", "power"),
  params
)
}
\arguments{
\item{n_units}{Number of test units (e.g. 5).}

\item{times}{Vector of inspection time points (e.g. \code{seq(72, 2160, by = 72)}).}

\item{stress_levels}{Vector of stress levels S = (S1, S2, ..., SK).}

\item{thresholds}{Vector of degradation thresholds (omega_1, ..., omega_K-1) at which stress is elevated.}

\item{process}{Character string specifying \code{"wiener"} or \code{"gamma"}.}

\item{model}{Acceleration model: \code{"arrhenius"} or \code{"power"}.}

\item{params}{Model parameters: \code{c(a, b, sigma)} for Wiener, or \code{c(a, b, u)} for Gamma.}
}
\value{
A data frame with columns \code{time}, \code{unit1}, \code{unit2}, ..., \code{unitN}.
}
\description{
Generates simulated degradation path data for N test units subject to a K-steps step-stress
accelerated degradation test (SSADT) under Wiener or Gamma stochastic process models.
}
\examples{
set.seed(42)
sim_data <- ssad_simulate(
  n_units = 5,
  times = seq(72, 2160, by = 72),
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  process = "wiener",
  model = "arrhenius",
  params = c(a = 5.3669, b = -2546.7, sigma = 0.1)
)
head(sim_data)
}
\references{
Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
\emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.ssadfit}
\alias{plot.ssadfit}
\title{Plot Diagnostics and Degradation Paths for SSADT Models}
\usage{
\method{plot}{ssadfit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"ssadfit"}.}

\item{...}{Additional graphical parameters.}
}
\value{
Invisibly returns the input object \code{x}. Side effect: displays diagnostic plots.
}
\description{
Generates diagnostic plots for a fitted step-stress degradation model, including observed vs. fitted
degradation paths and MCMC trace/posterior density plots (if fitted via MCMC).
}
\examples{
data(wiener_arrhenius)
fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
plot(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{geweke_diag}
\alias{geweke_diag}
\title{Geweke's MCMC Convergence Diagnostic}
\usage{
geweke_diag(chain, frac1 = 0.1, frac2 = 0.5)
}
\arguments{
\item{chain}{A numeric matrix or data frame where columns correspond to parameters and rows to iterations,
or a numeric vector for a single parameter.}

\item{frac1}{Fraction of the chain to take from the beginning (default: 0.1).}

\item{frac2}{Fraction of the chain to take from the end (default: 0.5).}
}
\value{
A data frame containing:
  \item{parameter}{Name of the parameter.}
  \item{mean_first}{Mean of the initial fraction of iterations.}
  \item{mean_last}{Mean of the final fraction of iterations.}
  \item{z_score}{Geweke z-statistic.}
  \item{p_value}{Two-sided p-value for equality of means.}
  \item{converged}{Logical indicating whether convergence is supported at alpha = 0.05 level.}
}
\description{
Computes Geweke's convergence diagnostic for MCMC chains by comparing the sample means of
early and late parts of the Markov chain.
}
\details{
Geweke's convergence diagnostic tests for equality of the mean of early simulations
(default first 10\%) and the mean of later iterations (default last 50\%). Under the null hypothesis
of convergence, the test statistic follows a standard normal distribution.
}
\examples{
set.seed(123)
sample_chain <- matrix(rnorm(1000), ncol = 2, dimnames = list(NULL, c("param1", "param2")))
diag_res <- geweke_diag(sample_chain)
print(diag_res)
}
\references{
Geweke, J. (1992). Evaluating the accuracy of sampling-based approaches to the calculation of posterior moments.
\emph{Bayesian Statistics}, 4, 169-193.

Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
\emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.ssadfit}
\alias{print.ssadfit}
\title{Print Method for Step-Stress Accelerated Degradation Fit}
\usage{
\method{print}{ssadfit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"ssadfit"}.}

\item{...}{Additional arguments passed to print.}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Prints a concise summary of the fitted step-stress accelerated degradation model.
}
\examples{
data(wiener_arrhenius)
fit <- ssad_fit(wiener_arrhenius, process = "wiener", model = "arrhenius", method = "mle")
print(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wiener_ssad.R
\name{ssad_wiener}
\alias{ssad_wiener}
\title{Wiener Process Step-Stress Accelerated Degradation Model Estimation}
\usage{
ssad_wiener(
  data,
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  model = c("arrhenius", "power"),
  method = c("mle", "mcmc"),
  init = NULL,
  n_iter = 5000,
  burnin = 1000,
  thin = 1
)
}
\arguments{
\item{data}{A data frame containing inspection times in the first column and unit degradation measurements in subsequent columns,
or a long-format data frame with columns \code{unit}, \code{time}, \code{degradation}.}

\item{stress_levels}{Vector of stress levels S = (S1, S2, ..., SK).}

\item{thresholds}{Vector of degradation thresholds (omega_1, ..., omega_K-1) at which stress levels are elevated.}

\item{model}{Acceleration model: \code{"arrhenius"} or \code{"power"}.}

\item{method}{Estimation method: \code{"mle"} or \code{"mcmc"}.}

\item{init}{Initial parameter values \code{c(a, b, sigma)}.}

\item{n_iter}{Number of MCMC iterations (default: 10000).}

\item{burnin}{Number of burn-in iterations (default: 2000).}

\item{thin}{Thinning interval for MCMC (default: 1).}
}
\value{
An object of class \code{"ssadfit"} containing fit results, parameter estimates, log-likelihood, MCMC samples (if applicable), and metadata.
}
\description{
Fits a Wiener process step-stress accelerated degradation model using Maximum Likelihood Estimation (MLE)
or Bayesian Markov Chain Monte Carlo (MCMC) as described in Pan and Balakrishnan (2010).
}
\examples{
data(wiener_arrhenius)
fit_w <- ssad_wiener(
  data = wiener_arrhenius,
  stress_levels = c(45, 65, 85),
  thresholds = c(90, 160),
  model = "arrhenius",
  method = "mle"
)
print(fit_w)
}
\references{
Pan, Z., & Balakrishnan, N. (2010). Multiple-Steps Step-Stress Accelerated Degradation Modeling Based on Wiener and Gamma Processes.
\emph{Communications in Statistics - Simulation and Computation}, 39(7), 1384-1402. \doi{10.1080/03610918.2010.496060}
}
